#include "testlib.h"
#include <iostream>
#include <vector>
#include <cassert>

using namespace std;

#define ll long long
#define pb emplace_back

const int INF = 1e9 + 10;

map<string, string> params;

long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}

const int MAXS = 200'000;
const int MAXN = 200'000;
const int MAXX = 1000'000'000;

ll C2(ll n) {
    return (n * (n - 1)) / 2;
}

ll count(ll x, ll k) {
    ll base_size = x / k;
    ll increased_size = x / k + 1;
    ll count_base_size = k - (x % k);
    ll count_increased_size = x % k;
    ll res = 0;
    res += C2(count_base_size) * base_size * base_size;
    res += C2(count_increased_size) * increased_size * increased_size;
    res += (base_size * count_base_size) * (increased_size * count_increased_size);
    return res;
}

vector<ll> solve(int n, ll b, vector<int> c) {
    vector<ll> cnt(MAXS + 5);
    vector<ll> add(MAXS + 5);
    for (int i = 0; i < n; i++) {
        for (int k = 1; k <= c[i]; k++) {
            cnt[k] += count(c[i], k);
        }
        add[c[i] + 1] += count(c[i], c[i]);
    }
    vector<ll> ans(MAXS + 5);
    ll y = 0;
    for (int k = 1; k < MAXS + 5; k++) {
        y += add[k];
        ans[k] = (cnt[k] + y) * b;
    }
    return ans;
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }
    int sum = get("sum", MAXS);
    int n = get("n", MAXN);
    int b = get("b", 1);
    int optk = get("optk", 1);
    int strong = get("strong", 0);
    vector<int> cnt;
    ll X = 0;
    if (strong == 1) {
        assert(n >= 10);
        int big = rnd.next(1, 10);
        cnt = rnd.partition(big, sum / 2, 1);
        sum -= sum / 2;
        vector<int> cnt1 = rnd.partition(n - big, sum, 1);
        for (auto &c : cnt1) {
            cnt.push_back(c);
        }
    } else {
        cnt = rnd.partition(n, sum, 1);
    }
    vector<ll> ans = solve(n, b, cnt);
    vector<ll> delta(ans.size());
    for (int k = 1; k < ans.size(); k++) {
        delta[k] = ans[k] - ans[k - 1];
    }
    X = delta[optk + 1] + 1;
    if (X > MAXX) {
        X = MAXX;
    }
    cout << n << " " << b << " " << X << "\n";
    shuffle(cnt.begin(), cnt.end());
    for (int i = 0; i < n; i++) {
        cout << cnt[i];
        if (i + 1 < n) {
            cout << " ";
        } else {
            cout << "\n";
        }
    }
    return 0;
}
