#include <cassert>
#include <initializer_list>
#include <numeric>
#include <vector>
#include <iostream>
#include <utility>
#include <stack>
#include <queue>
#include <set>
#include <map>
#include <algorithm>
#include <math.h>

using namespace std;

#define pii pair<int, int>
#define mp make_pair
#define fi first
#define se second
#define all(x) (x).begin(), (x).end()
#define ll long long
#define pb emplace_back

const int INF = 1e9 + 10;
const ll INFLL = 1e18;
const int mod = 1e9 + 7;

int C;

int add(int a, int b) {
    int res = a + b;
    if (res >= mod) {
        res -= mod;
    }
    return res;
}

int sub(int a, int b) {
    int res = a - b;
    if (res < 0) {
        res += mod;
    }
    return res;
}

int mul(int a, int b) {
    ll res = (ll)a * b;
    res %= mod;
    return res;
}

int Pow(int a, int b) {
    if (b == 0) {
        return 1;
    }
    ll res = Pow(a, b / 2);
    res = (res * res) % mod;
    if (b % 2 == 1) {
        res = (res * a) % mod;
    }
    return res;
}

int inv(int x) {
    return Pow(x, mod - 2);
}

class SetProduct {
public:
    SetProduct() = default;

    void AddElement(int x) {
        if (x == 0) {
            ++zeroes;
        } else {
            product = mul(product, x);
        }
    }

    void RemoveElement(int x) {
        if (x == 0) {
            assert(zeroes > 0);
            --zeroes;
        } else {
            product = mul(product, inv(x));
        }
    }

    int GetProduct() const {
        if (zeroes > 0) {
            return 0;
        } else {
            return product;
        }
    }

    int GetPositiveProduct() const {
        return product;
    }

private:
    size_t zeroes = 0;
    int product = 1;
};

pii count_boundries(const vector<int> &a, int pos) {
    int l = 0, r = C;
    if (a[pos] != -1) {
        l = a[pos];
        r = a[pos];
    }
    if (pos > 0 && a[pos - 1] != -1) {
        l = max(l, a[pos - 1]);
    }
    if (pos + 1 < a.size() && a[pos + 1] != -1) {
        r = min(r, a[pos + 1]);
    }
    return mp(l, r);
}

int count_formula(int n) {  // sum of abs(i, j) for all 0 <= i, j <= n
    n++;
    assert(n >= 1);
    if (n % 3 == 0) {
        return mul(n / 3, sub(mul(n, n), 1));
    } else {
        return mul(n, (((ll)n * n - 1) / 3) % mod);
    }
}

int count_formula2(int l, int r) {  // sum of i for all l <= i <= r
    ll sum = l + r;
    ll len = r - l + 1;
    return (sum * len / 2) % mod;
}

int count_sum_abs_segments_pair(const pii &x, const pii &y) {
    assert((x.fi == y.fi && x.se == y.se) || (x.se < y.fi) || (x.fi > y.se));
    if (x.fi == y.fi && x.se == y.se) {
        int n = x.se - x.fi;
        return count_formula(n);
    } else if (x.se < y.fi) {
        return sub(mul(count_formula2(y.fi, y.se), x.se - x.fi + 1),
                   mul(count_formula2(x.fi, x.se), y.se - y.fi + 1));
    } else {
        return sub(mul(count_formula2(x.fi, x.se), y.se - y.fi + 1),
                   mul(count_formula2(y.fi, y.se), x.se - x.fi + 1));
    }
}

int count_sum_abs(const pii &x, const pii &y) {
    if (x.fi > x.se || y.fi > y.se) {
        return 0;
    }
    pii intersect = mp(max(x.fi, y.fi), min(x.se, y.se));
    vector<pii> xsegments;
    vector<pii> ysegments;
    if (intersect.fi > intersect.se) {
        xsegments.pb(x);
        ysegments.pb(y);
    } else {
        xsegments.pb(intersect);
        ysegments.pb(intersect);
        if (x.fi < intersect.fi) {
            xsegments.pb(x.fi, intersect.fi - 1);
        }
        if (y.fi < intersect.fi) {
            ysegments.pb(y.fi, intersect.fi - 1);
        }
        if (x.se > intersect.se) {
            xsegments.pb(intersect.se + 1, x.se);
        }
        if (y.se > intersect.se) {
            ysegments.pb(intersect.se + 1, y.se);
        }
    }
    int res = 0;
    for (auto &l : xsegments) {
        for (auto &r : ysegments) {
            res = add(res, count_sum_abs_segments_pair(l, r));
        }
    }
    return res;
}

int TOTAL_SUM = 0;
SetProduct SET;

int calc_f(const vector<int> &a, const vector<int> &b, int i) {
    pii x = count_boundries(a, i);
    pii y = count_boundries(b, i);
    return count_sum_abs(x, y);
}

int calc_g(const vector<int> &a, const vector<int> &b, int i) {
    auto x = count_boundries(a, i);
    auto y = count_boundries(b, i);
    int g = mul(max(0, x.se - x.fi + 1), max(0, y.se - y.fi + 1));
    return g;
}

void solve_for_fixed_arrays(const vector<int> &a, const vector<int> &b) {
    int n = a.size();
    for (int i = 0; i < n; i++) {
        int g = calc_g(a, b, i);
        SET.AddElement(g);
    }
    for (int i = 0; i < n; i++) {
        int g = calc_g(a, b, i);
        // SET.RemoveElement(g); // bug
        int coef = SET.GetPositiveProduct();
        TOTAL_SUM = add(TOTAL_SUM, mul(calc_f(a, b, i), coef));
        SET.AddElement(g);
    }
}

void print_answer() {
    if (SET.GetProduct() == 0) {
        cout << 0 << "\n";
    } else {
        cout << TOTAL_SUM << "\n";
    }
}

void solve() {
    int n, q;
    cin >> n >> C >> q;
    vector<int> a(n);
    for (auto &c : a) {
        cin >> c;
    }
    vector<int> b(n);
    for (auto &c : b) {
        cin >> c;
    }
    solve_for_fixed_arrays(a, b);
    print_answer();
    while (q--) {
        int t, p, x;
        cin >> t >> p >> x;
        p--;
        int preva = a[p], prevb = b[p];
        for (int pos = max(0, p - 1); pos <= min(n - 1, p + 1); pos++) {
            int f = calc_f(a, b, pos);
            int g = calc_g(a, b, pos);
            SET.RemoveElement(g);
            TOTAL_SUM = sub(TOTAL_SUM, mul(f, SET.GetPositiveProduct()));
            TOTAL_SUM = mul(TOTAL_SUM, inv(max(1, g)));
            if (t == 1) {
                a[p] = x;
            } else {
                b[p] = x;
            }
            f = calc_f(a, b, pos);
            g = calc_g(a, b, pos);
            TOTAL_SUM = mul(TOTAL_SUM, max(g, 1));
            TOTAL_SUM = add(TOTAL_SUM, mul(f, SET.GetPositiveProduct()));
            SET.AddElement(g);
            a[p] = preva;
            b[p] = prevb;
        }
        if (t == 1) {
            a[p] = x;
        } else {
            b[p] = x;
        }
        print_answer();
    }
}

int main() {
    cin.tie(0);
    cout.tie(0);
    ios_base::sync_with_stdio(0);
    int t = 1;
    // cin >> t;
    while (t--) {
        solve();
    }
    return 0;
}
