#include "testlib.h"
#include <iostream>
#include <cassert>

using namespace std;

map<string, string> parse;

int num(string &s) {
    return stoi(s);
}


int n = 300'000;
vector<int> RES;
int minc = -1'000'000'000;
int maxc = 1'000'000'000;


void gen_random() {
    if (parse.count("n")) n = num(parse["n"]);
    if (parse.count("minc")) minc = num(parse["minc"]);
    if (parse.count("maxc")) maxc = num(parse["maxc"]);
    RES.resize(n);
    for (int i = 0; i < n; i++)
        RES[i] = rnd.next(minc, maxc);
}

void gen_random_smarter() {
    if (parse.count("n")) n = num(parse["n"]);
    if (parse.count("minc")) minc = num(parse["minc"]);
    if (parse.count("maxc")) maxc = num(parse["maxc"]);
    int max_len = n;
    if (parse.count("len")) max_len = num(parse["len"]);
    RES.resize(n);
    int pt = 0;
    int iters = max(2, static_cast<int32_t>(pow(max_len, 0.33)));
    if (maxc - minc + 1 < max_len)
        throw std::runtime_error("value range shorter than max_len");
    while (pt < n) {
        int rem = n - pt;
        int len = rnd.wnext(1, min(rem, max_len), iters);
        RES[pt] = rnd.next(minc + len - 1, maxc);
        for (int j = pt + 1; j < pt + len; j++)
            RES[j] = RES[j - 1] - 1;
        pt += len;
    }
    int res = -1, resid = -1;
    for (int i = 0; i < n;) {
        int j = i + 1;
        while (j < n && RES[j] < RES[i]) j++;
        if (res < j - i) {
            resid = i;
            res = j - i;
        }
        i = j;
    }
    int strong_order_ct = 0;
    if (parse.count("strong_order_ct")) {
        strong_order_ct = num(parse["strong_order_ct"]);
    }
    if (strong_order_ct) {
        if (res + 1 < resid) {
            for (int k = 0; k < res + 1; k++)
                RES[k] = -(1 << 30);
        } else {
            for (int k = resid + res + 1; k < min(n, resid + res + 1 + res + 2); k++)
                RES[k] = -(1 << 30);
        }
    }
}

void gen_random_smarter_strong() {
    if (parse.count("n")) n = num(parse["n"]);
    if (parse.count("minc")) minc = num(parse["minc"]);
    if (parse.count("maxc")) maxc = num(parse["maxc"]);
    vector<int> lens;
    int max_len = n;
    if (parse.count("len")) max_len = num(parse["len"]);
    RES.resize(n);
    int iters = max(2, static_cast<int32_t>(pow(max_len, 0.33)));
    if (maxc - minc + 1 < max_len)
        throw std::runtime_error("value range shorter than max_len");
    int rem = n;
    while (rem > 0) {
        int len = rnd.wnext(1, min(rem, max_len), iters);
        lens.push_back(len);
        rem -= len;
    }
    sort(lens.begin(), lens.end());
//    shuffle(lens.begin(), lens.end());
    int pt = 0, last_mx = minc + 1;
    for (auto len: lens) {
        last_mx += len;
        int cur = last_mx;
        for (int j = pt; j < pt + len; j++, cur--) {
            RES[j] = cur;
        }
        last_mx++;
        pt += len;
    }
    int res = -1, resid = -1;
    for (int i = 0; i < n;) {
        int j = i + 1;
        while (j < n && RES[j] < RES[i]) j++;
        if (res < j - i) {
            resid = i;
            res = j - i;
        }
        i = j;
    }
    int strong_order_ct = 0;
    if (parse.count("strong_order_ct")) {
        strong_order_ct = num(parse["strong_order_ct"]);
    }
    if (strong_order_ct) {
        if (res + 1 < resid) {
            for (int k = 0; k < res + 1; k++)
                RES[k] = -(1 << 30);
        } else {
            for (int k = resid + res + 1; k < min(n, resid + res + 1 + res + 2); k++)
                RES[k] = -(1 << 30);
        }
    }
}

void build_increasing() {
    if (parse.count("n")) n = num(parse["n"]);
    RES.resize(n);
    int chance = 1000000000;
    if (parse.count("chance")) chance = num(parse["chance"]);
    int pt = 0;
    for (int i = 0; i < n; i++) {
        RES[i] = pt;
        pt++;
        if (rnd.next(1, chance) == 1) pt--;
    }
}

void build_decreasing() {
    if (parse.count("n")) n = num(parse["n"]);
    RES.resize(n);
    int chance = 1000000000;
    if (parse.count("chance")) chance = num(parse["chance"]);
    int pt = n;
    for (int i = 0; i < n; i++) {
        RES[i] = pt;
        pt--;
        if (rnd.next(1, chance) == 1) pt++;
    }
}

int calc_strong() {
    int res = -1, resid = -1;
    for (int i = 0; i < n;) {
        int j = i + 1;
        while (j < n && RES[j] < RES[i]) j++;
        res = max(res, j - i - 1);
        i = j;
    }
    return res;
}

int calc_not_strong() {
    int res = -1, resid = -1;
    for (int i = 0; i < n;) {
        int j = i + 1;
        while (j < n && RES[j] <= RES[i]) j++;
        res = max(res, j - i - 1);
        i = j;
    }
    return res;
}

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string cur = argv[i];
        int p = find(cur.begin(), cur.end(), '=') - cur.begin();
        parse[cur.substr(0, p)] = cur.substr(p + 1, (int) cur.size() - p);
    }
    if (parse["mode"] == "random") {
        gen_random();
    } else if (parse["mode"] == "random_smart") {
        int strong_order_ct = 0;
        if (parse.count("strong_order_ct")) {
            strong_order_ct = num(parse["strong_order_ct"]);
        }
        while (true) {
            gen_random_smarter();
            if (!strong_order_ct) break;
            else {
                if (calc_not_strong() != calc_strong()) break;
            }
        }
    } else if (parse["mode"] == "increasing") {
        build_increasing();
    } else if (parse["mode"] == "decreasing") {
        build_decreasing();
    } else if (parse["mode"] == "random_smart_strong") {
        int strong_order_ct = 0;
        if (parse.count("strong_order_ct")) {
            strong_order_ct = num(parse["strong_order_ct"]);
        }
        while (true) {
            gen_random_smarter_strong();
            if (!strong_order_ct) break;
            else {
                if (calc_not_strong() != calc_strong()) break;
            }
        }
    }

    cout << RES.size() << '\n';
    vector<int> cord = RES;
    sort(cord.begin(), cord.end());
    cord.resize(unique(cord.begin(), cord.end()) - cord.begin());
    set<int> values;
    while (values.size() < cord.size()) {
        values.insert(rnd.next(minc, maxc));
    }
    vector<int> vals(values.begin(), values.end());
    for (auto &i: RES)
        i = vals[lower_bound(cord.begin(), cord.end(), i) - cord.begin()]; // randomize input values
    for (int i = 0; i < RES.size(); i++) {
        cout << RES[i];
        if (i + 1 < RES.size()) cout << ' ';
    }
    cout << endl;
}
