#include <bits/stdc++.h>

using namespace std;
using ll = long long;
using pi = pair<int, int>;

struct SegmentTree {
    int n;
    vector<ll> t;

    SegmentTree(int n) : n(n), t(2*n) { }

    void Add(int i, ll x) {
        for (i += n; i != 0; i >>= 1) t[i] += x;
    }

    ll Query(int l, int r) {
        ll ans = 0;

        for (l += n, r += n - 1; l <= r; l >>= 1, r >>= 1) {
            if ((l&1) == 1) ans += t[l++];
            if ((r&1) == 0) ans += t[r--];
        }
        return ans;
    }
};

struct SegmentContainer {
    int side;
    SegmentTree sgt_a, sgt_b, sgt_c, sgt_d;
    int id;
    // sgt_a: sum(1)
    // sgt_b: sum(m)
    // sgt_c: sum(m*(m+1)/2)
    // sgt_d: sum(m*(m-1)*(2*m-1)/6 - m*(m-1)/2*m)

    SegmentContainer(int side) : side(side), sgt_a(side), sgt_b(side), sgt_c(side), sgt_d(side) {
    }

    tuple<ll, ll, ll, ll> GetABCD(ll m) {
        return make_tuple(1, m, m*(m+1)/2, m*(m-1)*(2*m-1)/6 - m*(m-1)/2*m);
    }

    void Insert(int m) {
        auto [a, b, c, d] = GetABCD(m);
        sgt_a.Add(m-1, +a);
        sgt_b.Add(m-1, +b);
        sgt_c.Add(m-1, +c);
        sgt_d.Add(m-1, +d);
    }

    void Erase(int m) {
        auto [a, b, c, d] = GetABCD(m);
        sgt_a.Add(m-1, -a);
        sgt_b.Add(m-1, -b);
        sgt_c.Add(m-1, -c);
        sgt_d.Add(m-1, -d);
    }

    ll SquaresCount(int n) {
        const int mid = min(side, n);
        auto sum_a = sgt_a.Query(mid, side); // m > n
        auto sum_b = sgt_b.Query(mid, side); // m > n
        auto sum_c = sgt_c.Query(0, mid); // m <= n
        auto sum_d = sgt_d.Query(0, mid); // m <= n
        auto [a, b, c, d] = GetABCD(n);

        return d * sum_a + c * sum_b + b * sum_c + a * sum_d;
    }
};

struct SegmentMaintainer {
    SegmentContainer& my;
    SegmentContainer& other;
    ll& ans;

    set<int> pos_zero;
    vector<ll> diff_array;

    SegmentMaintainer(vector<int> a, 
                      SegmentContainer& my,
                      SegmentContainer& other,
                      ll& ans) :
            pos_zero(), diff_array(a.size()), my(my), other(other), ans(ans) {
        pos_zero.insert(0);
        for (int i = 1; i < a.size(); ++i) {
            diff_array[i] = a[i] - a[i-1];
            if (diff_array[i] == 0) pos_zero.insert(i);
        }
        pos_zero.insert(a.size());

        for (auto it = pos_zero.begin(); *it != my.side; ++it) {
            OnSegmentAppear(*next(it) - *it);
        }
    }

    void OnSegmentAppear(int len) {
        my.Insert(len);
        ans += other.SquaresCount(len);
    }

    void OnSegmentDissapear(int len) {
        my.Erase(len);
        ans -= other.SquaresCount(len);
    }

    void ChangeBound(int pos, ll dx) {
        if (pos == 0 || pos == my.side) return; 
            
        bool was_zero = diff_array[pos] == 0;
        diff_array[pos] += dx;
        bool now_zero = diff_array[pos] == 0;

        if (was_zero && !now_zero) {
            auto mid = pos_zero.find(pos);
            auto prv = prev(mid), nxt = next(mid);
            
            OnSegmentDissapear(*mid - *prv);
            OnSegmentDissapear(*nxt - *mid);
            OnSegmentAppear(*nxt - *prv);

            pos_zero.erase(mid);
        }
        
        if (!was_zero && now_zero) {
            auto mid = pos_zero.insert(pos).first;
            auto prv = prev(mid), nxt = next(mid);

            OnSegmentAppear(*mid - *prv);
            OnSegmentAppear(*nxt - *mid);
            OnSegmentDissapear(*nxt - *prv);
        }
    }

    void RangeAdd(int l, int r, int x) {
        ChangeBound(l, +x);
        ChangeBound(r, -x);
    }
};

int main() {
    ios::sync_with_stdio(0); cin.tie(0);
    int n, m, q; cin >> n >> m >> q;

    vector<int> a(n), b(m);
    for (int& x : a) cin >> x;
    for (int& x : b) cin >> x;

    ll ans = 0;
    SegmentContainer a_segments(n), b_segments(m);

    a_segments.id = 1;
    b_segments.id = 2;

    SegmentMaintainer a_maintainer(a, a_segments, b_segments, ans); 
    SegmentMaintainer b_maintainer(b, b_segments, a_segments, ans);
    
    cout << ans << '\n';
    while (q--) {
        int t, l, r, x; cin >> t >> l >> r >> x; --l;
        if (t == 1) a_maintainer.RangeAdd(l, r, x);
        if (t == 2) b_maintainer.RangeAdd(l, r, x);
        cout << ans << '\n';
    }
}