#pragma GCC optimize("O3")
#pragma GCC target("avx2")

#include <bits/stdc++.h>

using namespace std;
using ll = long long;
using pi = pair<int, int>;

constexpr int N = 300'228;

ll a[N], b[N], c[N], d[N];

void init() {
    for (int m = 1; m < N; ++m) {
        a[m] = 1;
        b[m] = m;
        c[m] = (ll)m*(m+1)/2;
        d[m] = (ll)m*(m-1)*(2*m-1)/6 - (ll)m*(m-1)/2*m;
    }
}

static const int BOUND = 800;

struct SegmentContainer {

    int side, utimer, dynamic_bound;
    vector<int> cnt, was, w;
    vector<ll> B, C, D;

    SegmentContainer(int side) : side(side), utimer(0), dynamic_bound(0), cnt(side+1), was(side+1), w{},
        B(side+1), C(side+1), D(side+1) {
    }

    void Insert(int m) {
        if (!cnt[m]) {
            if (m > BOUND) w.push_back(m);
            else dynamic_bound = max(dynamic_bound, m);
        }
        ++cnt[m];
        B[m] += m;
        C[m] += c[m];
        D[m] += d[m];
    }

    void Erase(int m) {
        --cnt[m];
        B[m] -= m;
        C[m] -= c[m];
        D[m] -= d[m];
    }

    ll SquaresCount(int n) {
        ++utimer;
        while (cnt[dynamic_bound] == 0 && dynamic_bound != 0) --dynamic_bound;

        const auto& A = cnt;
        ll sA = n <= dynamic_bound ? accumulate(A.begin()+n, A.begin()+dynamic_bound+1, 0ll) : 0ll;
        ll sB = n <= dynamic_bound ? accumulate(B.begin()+n, B.begin()+dynamic_bound+1, 0ll) : 0ll;
        ll sC = accumulate(C.begin(), C.begin()+min(n, dynamic_bound+1), 0ll);
        ll sD = accumulate(D.begin(), D.begin()+min(n, dynamic_bound+1), 0ll);

        for (int i = 0; i < w.size(); ) {
            if (was[w[i]] == utimer || cnt[w[i]] == 0) {
                swap(w[i], w.back());
                w.pop_back();
            } else {
                was[w[i]] = utimer;
                if (w[i] < n) {
                    sD += D[w[i]];
                    sC += C[w[i]];
                } else {
                    sA += cnt[w[i]];
                    sB += B[w[i]];
                }
                ++i;
            }
        }

        return sD + n * sC + c[n] * sB + d[n] * sA;
    }
};

struct SegmentMaintainer {
    SegmentContainer& my;
    SegmentContainer& other;
    ll& ans;

    set<int> pos_zero;
    vector<ll> diff_array;

    SegmentMaintainer(vector<int> a, 
                      SegmentContainer& my,
                      SegmentContainer& other,
                      ll& ans) :
            pos_zero(), diff_array(a.size()), my(my), other(other), ans(ans) {
        pos_zero.insert(0);
        for (int i = 1; i < a.size(); ++i) {
            diff_array[i] = a[i] - a[i-1];
            if (diff_array[i] == 0) pos_zero.insert(i);
        }
        pos_zero.insert(a.size());

        for (auto it = pos_zero.begin(); *it != my.side; ++it) {
            OnSegmentAppear(*next(it) - *it);
        }
    }

    void OnSegmentAppear(int len) {
        my.Insert(len);
        ans += other.SquaresCount(len);
    }

    void OnSegmentDissapear(int len) {
        my.Erase(len);
        ans -= other.SquaresCount(len);
    }

    void ChangeBound(int pos, ll dx) {
        if (pos == 0 || pos == my.side) return; 
            
        bool was_zero = diff_array[pos] == 0;
        diff_array[pos] += dx;
        bool now_zero = diff_array[pos] == 0;

        if (was_zero && !now_zero) {
            auto mid = pos_zero.find(pos);
            auto prv = prev(mid), nxt = next(mid);
            
            OnSegmentDissapear(*mid - *prv);
            OnSegmentDissapear(*nxt - *mid);
            OnSegmentAppear(*nxt - *prv);

            pos_zero.erase(mid);
        }
        
        if (!was_zero && now_zero) {
            auto mid = pos_zero.insert(pos).first;
            auto prv = prev(mid), nxt = next(mid);

            OnSegmentAppear(*mid - *prv);
            OnSegmentAppear(*nxt - *mid);
            OnSegmentDissapear(*nxt - *prv);
        }
    }

    void RangeAdd(int l, int r, int x) {
        ChangeBound(l, +x);
        ChangeBound(r, -x);
    }
};

int main() {
    ios::sync_with_stdio(0); cin.tie(0);
    init();
    int n, m, q; cin >> n >> m >> q;

    vector<int> a(n), b(m);
    for (int& x : a) cin >> x;
    for (int& x : b) cin >> x;

    ll ans = 0;
    SegmentContainer a_segments(n), b_segments(m);
    SegmentMaintainer a_maintainer(a, a_segments, b_segments, ans); 
    SegmentMaintainer b_maintainer(b, b_segments, a_segments, ans);
    
    cout << ans << '\n';
    while (q--) {
        int t, l, r, x; cin >> t >> l >> r >> x; --l;
        if (t == 1) a_maintainer.RangeAdd(l, r, x);
        if (t == 2) b_maintainer.RangeAdd(l, r, x);
        cout << ans << '\n';
    }
}